package jp.sourceforge.talisman.mds.cluster;

/*
 * $Id: SequentialHardCMeanMethod.java 90 2008-06-28 02:25:49Z tama3 $
 */

import jp.sourceforge.talisman.mds.Item;
import jp.sourceforge.talisman.mds.distance.ItemDistanceCalculator;

/**
 * Sequential hard(Crisp) c-Mean algorithm.
 *
 * @author Haruaki Tamada
 * @version $Revision: 90 $ 
 */
class SequentialHardCMeanMethod extends NonHierarchicalClusteringMethod{
    public SequentialHardCMeanMethod(ClusteringParameter param) throws ParameterTypeMismatchException{
        super(param);
        if(!(param instanceof NonHierarchicalClusteringParameter)){
            throw new ParameterTypeMismatchException("NonHierarchicalClusteringParameter is required.");
        }
    }

    public SequentialHardCMeanMethod(NonHierarchicalClusteringParameter param){
        super(param);
    }

    @Override
    protected Cluster[] doClustering(Item[] items, ClusteringParameter initParam){
        NonHierarchicalClusteringParameter param = (NonHierarchicalClusteringParameter)initParam;

        Cluster[] clusters = initClusters(items, param);
        ItemDistanceCalculator calculator = param.getDistanceCalculator();

        boolean flag = true;
        while(flag){
            flag = false;
            for(int i = 0; i < items.length; i++){
                int current = -1;
                int target = -1;
                double length = Double.MAX_VALUE;
                for(int j = 0; j < clusters.length; j++){
                    if(clusters[j].isContain(items[i])){
                        current = j;
                    }
                    double distance = calculator.calculate(clusters[j].getMedian(), items[i]);
                    if(length > distance){
                        length = distance;
                        target = j;
                    }
                }
                clusters[current].removeItem(items[i]);
                clusters[target].addItem(items[i]);
                flag = flag || current != target;
            }
        }

        return clusters;
    }
}
