package jp.sourceforge.talisman.mds.distance;

/*
 * $Id: SquaredEuclideanItemDistanceCalculator.java 90 2008-06-28 02:25:49Z tama3 $
 */

/**
 * Squared euclidean distance calculator.
 * <pre>
 * D(x, y) = \sigma^r_{k=1} (x_k - y_k)^2
 * </pre>
 *
 * @author Haruaki Tamada
 * @version $Revision: 90 $ 
 */
public class SquaredEuclideanItemDistanceCalculator extends AbstractItemDistanceCalculator{
    @Override
    public ItemDistanceAlgorithm getAlgorithm(){
        return ItemDistanceAlgorithm.SQUARED_EUCLIDEAN_DISTANCE;
    }

    @Override
    public double calculate(double[] x, double[] y){
        validateArguments(x, y);

        double value = 0d;
        for(int i = 0; i < x.length; i++){
            if(!Double.isNaN(x[i]) && !Double.isNaN(y[i])){
                value += (x[i] - y[i]) * (x[i] - y[i]);
            }
        }

        return Math.sqrt(value);
    }
}
