package jp.sourceforge.talisman.mds.ui.mark;

/*
 * $Id: DrawerFactory.java 158 2008-07-15 10:45:01Z tama3 $
 */

import java.util.LinkedHashMap;
import java.util.Map;

import javax.swing.Icon;

import jp.sourceforge.talisman.mds.ui.swing.GeometoryType;
import jp.sourceforge.talisman.mds.ui.swing.MarkDrawer;

/**
 * 
 * @author Haruaki Tamada
 * @version $Revision: 158 $ 
 */
public class DrawerFactory{
    private static final DrawerFactory instance = new DrawerFactory();

    private Map<GeometoryType, MarkDrawer> drawers = new LinkedHashMap<GeometoryType, MarkDrawer>();

    private DrawerFactory(){
        drawers.put(GeometoryType.XMARK,                  new XMarkDrawer());
        drawers.put(GeometoryType.CROSS,                  new CrossDrawer());
        drawers.put(GeometoryType.STAR,                   new StarDrawer());
        drawers.put(GeometoryType.CIRCLE,                 new CircleDrawer());
        drawers.put(GeometoryType.FILLED_CIRCLE,          new CircleDrawer(true));
        drawers.put(GeometoryType.RHOMBUS,                new RhombusDrawer());
        drawers.put(GeometoryType.FILLED_RHOMBUS,         new RhombusDrawer(true));
        drawers.put(GeometoryType.RECTANGLE,              new RectangleDrawer());
        drawers.put(GeometoryType.FILLED_RECTANGLE,       new RectangleDrawer(true));
        drawers.put(GeometoryType.UPPER_TRIANGLE,         new UpperTriangleDrawer());
        drawers.put(GeometoryType.FILLED_UPPER_TRIANGLE,  new UpperTriangleDrawer(true));
        drawers.put(GeometoryType.DOWNER_TRIANGLE,        new DownerTriangleDrawer());
        drawers.put(GeometoryType.FILLED_DOWNER_TRIANGLE, new DownerTriangleDrawer(true));
    }

    public static DrawerFactory getInstance(){
        return instance;
    }

    public MarkDrawer create(int type){
        GeometoryType[] types = GeometoryType.values();
        if(type < (types.length - 1)){
            return create(types[type + 1]);
        }
        return create(types[0]);
    }

    public MarkDrawer create(GeometoryType type){
        return drawers.get(type);
    }

    public Icon createIcon(int type){
        GeometoryType[] types = GeometoryType.values();
        if(type < (types.length - 1)){
            return createIcon(types[type + 1]);
        }
        return createIcon(types[0]);
    }

    public Icon createIcon(GeometoryType type){
        return new MarkIcon(create(type));
    }
}
