package jp.sourceforge.talisman.mds.ui.swing.actions;

/*
 * $Id: ClearAction.java 72 2008-01-16 05:55:21Z tama3 $
 */

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.util.HashMap;
import java.util.Map;

import javax.swing.AbstractAction;
import javax.swing.Icon;
import javax.swing.JComboBox;
import javax.swing.JFormattedTextField;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

import jp.sourceforge.talisman.mds.cluster.ClusteringAlgorithm;
import jp.sourceforge.talisman.mds.cluster.ClusteringParameter;
import jp.sourceforge.talisman.mds.cluster.HierarchicalClusteringParameter;
import jp.sourceforge.talisman.mds.cluster.NonHierarchicalClusteringParameter;
import jp.sourceforge.talisman.mds.distance.ClusterDistanceAlgorithm;
import jp.sourceforge.talisman.mds.distance.ItemDistanceAlgorithm;
import jp.sourceforge.talisman.mds.ui.swing.MdsPane;

/**
 * 
 * @author Haruaki Tamada
 * @version $Revision: 72 $ 
 */
public class ClusteringAction extends AbstractAction{
    private static final long serialVersionUID = -4431502090699769330L;
    private MdsPane mds;

    public ClusteringAction(MdsPane mds){
        super(mds.getMessages().get("clustering.label"));
        if(mds.getMessages().hasValue("clustering.tooltip")){
            putValue(SHORT_DESCRIPTION, mds.getMessages().get("clustering.tooltip"));
        }
        if(mds.getMessages().hasValue("clustering.icon")){
            Icon icon = mds.getMessages().getIcon("clustering.icon");
            putValue(SMALL_ICON, icon);
        }
        this.mds = mds;
    }

    public void actionPerformed(ActionEvent e){
        ClusteringSettingPane pane = new ClusteringSettingPane(mds);
        int flag = JOptionPane.showConfirmDialog(
            mds, pane, mds.getMessages().get("clustering.setting.title"),
            JOptionPane.OK_CANCEL_OPTION, JOptionPane.INFORMATION_MESSAGE
        );
        if(flag == JOptionPane.OK_OPTION){
            ClusteringParameter param = null;
            try{
                param = createParameter(pane);
            } catch(IllegalArgumentException exception){
                JOptionPane.showMessageDialog(
                    pane, exception.getMessage(), mds.getMessages().get("title.illegal.argument"),
                    JOptionPane.ERROR_MESSAGE
                );
            }
            if(param != null){
                mds.clustering(pane.getClusteringAlgorithm(), param);
            }
        }
    }

    private ClusteringParameter createParameter(ClusteringSettingPane pane){
        ClusteringAlgorithm algorithm = pane.getClusteringAlgorithm();
        ClusteringParameter param = null;
        switch(algorithm){
        case EXCHANGE_METHOD:
        case PARALLEL_HCM:
        case SEQUENTIAL_HCM:
            param = new NonHierarchicalClusteringParameter(pane.getClusterSize());
            param.setDistanceCalculator(pane.getItemDistanceCalculatorAlgorithm());
            break;
        case HIERARCHICAL_METHOD:
        case LANCE_AND_WILLIAMS_METHOD:
            HierarchicalClusteringParameter hcParam = new HierarchicalClusteringParameter(pane.getClusterSize());
            hcParam.setDistanceCalculator(pane.getItemDistanceCalculatorAlgorithm());
            hcParam.setClusterDistanceCalculator(pane.getClusterDistanceCalculatorAlgorithm());
            param = hcParam;
            break;
        default:
            throw new IllegalArgumentException("unknown algorithm: " + algorithm);
        }
        return param;
    }

    private static class ClusteringSettingPane extends JPanel{
        private static final long serialVersionUID = 4661816651445878479L;
        private MdsPane mds;
        private JFormattedTextField text;
        private JComboBox itemDistanceCombo;
        private JComboBox clusteringMethodCombo;
        private JComboBox clusterDistanceCombo;
        private Map<ClusteringAlgorithm, String> cAlgorithms = new HashMap<ClusteringAlgorithm, String>();
        private Map<ItemDistanceAlgorithm, String> idAlgorithms = new HashMap<ItemDistanceAlgorithm, String>();
        private Map<ClusterDistanceAlgorithm, String> cdAlgorithms = new HashMap<ClusterDistanceAlgorithm, String>();

        public ClusteringSettingPane(MdsPane mds){
            this.mds = mds;
            clusteringMethodCombo = createCombo(ClusteringAlgorithm.values(), "ca.label.", cAlgorithms);
            clusteringMethodCombo.setBorder(new TitledBorder(mds.getMessages().get("clusteringalgorithm.label")));

            itemDistanceCombo = createCombo(ItemDistanceAlgorithm.values(), "ida.label.", idAlgorithms);
            itemDistanceCombo.setBorder(new TitledBorder(mds.getMessages().get("itemdistancecalculator.label")));

            clusterDistanceCombo = createCombo(ClusterDistanceAlgorithm.values(), "cda.label.", cdAlgorithms);
            clusterDistanceCombo.setBorder(new TitledBorder(mds.getMessages().get("clusterdistancecalculator.label")));

            text = new JFormattedTextField();
            text.setBorder(new TitledBorder(mds.getMessages().get("clustering.cluster.size")));
            text.setValue(new Integer(3));

            setLayout(new GridLayout(4, 1));
            add(clusteringMethodCombo);
            add(itemDistanceCombo);
            add(clusterDistanceCombo);
            add(text);
        }

        public ItemDistanceAlgorithm getItemDistanceCalculatorAlgorithm(){
            return getAlgorithm(idAlgorithms, itemDistanceCombo);
        }

        public ClusteringAlgorithm getClusteringAlgorithm(){
            return getAlgorithm(cAlgorithms, clusteringMethodCombo);
        }

        public ClusterDistanceAlgorithm getClusterDistanceCalculatorAlgorithm(){
            return getAlgorithm(cdAlgorithms, clusterDistanceCombo);
        }

        public int getClusterSize(){
            return ((Integer)text.getValue()).intValue();
        }

        private <T> T getAlgorithm(Map<T, String> map, JComboBox combo){
            String label = (String)combo.getSelectedItem();
            T algorithm = null;
            for(Map.Entry<T, String> entry: map.entrySet()){
                if(label.equals(entry.getValue())){
                    algorithm = entry.getKey();
                    break;
                }
            }
            return algorithm;
        }

        private <T extends Enum<?>> JComboBox createCombo(T[] algorithms, String prefix, Map<T, String> map){
            JComboBox combo = new JComboBox();
            for(T algorithm: algorithms){
                String label = mds.getMessages().get(prefix + algorithm.name());
                map.put(algorithm, label);
                combo.addItem(label);
            }
            return combo;
        }
    };
}
