package jp.sourceforge.talisman.mds.ui.swing.actions;

/*
 * $Id: ShowItemsAction.java 158 2008-07-15 10:45:01Z tama3 $
 */

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Box;
import javax.swing.Icon;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;

import jp.sourceforge.talisman.mds.Item;
import jp.sourceforge.talisman.mds.MdsMethod;
import jp.sourceforge.talisman.mds.ui.swing.MdsPane;

/**
 * 
 * @author Haruaki Tamada
 * @version $Revision: 158 $ 
 */
public class ShowItemsAction extends AbstractAction{
    private static final long serialVersionUID = 445562293718912259L;

    private MdsPane mds;
    private Component parent;

    public ShowItemsAction(MdsPane mds){
        this(mds, mds);
    }

    public ShowItemsAction(Component parent, MdsPane mds){
        super(mds.getMessages().get("show.coordinates.label"));
        this.parent = parent;
        this.mds = mds;
        if(mds.getMessages().hasValue("show.coordinates.tooltip")){
            putValue(SHORT_DESCRIPTION, mds.getMessages().get("show.coordinates.tooltip"));
        }
        if(mds.getMessages().hasValue("show.coordinates.icon")){
            Icon icon = mds.getMessages().getIcon("show.coordinates.icon");
            putValue(SMALL_ICON, icon);
        }
    }

    public void actionPerformed(ActionEvent e){
        JOptionPane.showMessageDialog(
            parent, createPane(), mds.getMessages().get("show.coordinates.title"),
            JOptionPane.INFORMATION_MESSAGE
        );
    }

    private Component createPane(){
        DefaultTableModel model = new UneditableDefaultTableModel();
        TableSorter sorter = new TableSorter(model);

        MdsMethod<?> method = mds.getMdsMethod();
        String[] identifiers = new String[method.getRank() + 1];
        identifiers[0] = "";
        for(int i = 1; i < identifiers.length; i++){
            identifiers[i] = mds.getMessages().get("show.coordinate.xaxis.label") + i;
        }
        model.setColumnIdentifiers(identifiers);
        Item[] items = method.getItems();
        for(int i = 0; i < items.length; i++){
            String[] values = new String[identifiers.length];
            values[0] = items[i].getName();
            for(int j = 1; j < values.length; j++){
                values[j] = String.valueOf(items[i].get(j - 1));
            }
            model.addRow(values);
        }
        final JTable table = new JTable(sorter);
        sorter.setTableHeader(table.getTableHeader());
        JScrollPane scroll = new JScrollPane(table, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
        scroll.setSize(new Dimension(300, 300));

        Box north = Box.createHorizontalBox();
        north.add(Box.createHorizontalGlue());
        north.add(new JLabel(String.format("%s: %d", mds.getMessages().get("show.coordinate.rank.label"), method.getRank())));
        north.add(Box.createHorizontalGlue());

        JPanel panel = new JPanel(new BorderLayout());
        panel.add(scroll, BorderLayout.CENTER);
        panel.add(north, BorderLayout.NORTH);

        return panel;
    }
}
