package jp.sourceforge.talisman.mds.ui.swing.actions;

/*
 * $Id: StoreMdsImageAction.java 158 2008-07-15 10:45:01Z tama3 $
 */

import java.awt.Component;
import java.awt.Container;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.AbstractAction;
import javax.swing.Icon;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;

import jp.sourceforge.talisman.mds.ui.swing.CurrentDirectoryHandler;
import jp.sourceforge.talisman.mds.ui.swing.MdsPane;
import jp.sourceforge.talisman.mds.ui.swing.MdsViewerPane;

/**
 * 
 * @author Haruaki Tamada
 * @version $Revision: 158 $ 
 */
public class StoreMdsImageAction extends AbstractAction{
    private static final long serialVersionUID = 445562293718912259L;

    private MdsViewerPane mds;
    private CurrentDirectoryHandler handler;
    private Component parent;

    public StoreMdsImageAction(MdsViewerPane mds){
        this(mds, mds, mds);
    }

    public StoreMdsImageAction(Component parent, MdsViewerPane mds){
        this(parent, mds, mds);
    }

    public StoreMdsImageAction(Component parent, MdsViewerPane mds, CurrentDirectoryHandler handler){
        super(mds.getMessages().get("store.image.label"));
        this.parent = parent;
        this.mds = mds;
        this.handler = handler;

        if(mds.getMessages().hasValue("store.image.tooltip")){
            putValue(SHORT_DESCRIPTION, mds.getMessages().get("store.image.tooltip"));
        }
        if(mds.getMessages().hasValue("store.image.icon")){
            Icon icon = mds.getMessages().getIcon("store.image.icon");
            putValue(SMALL_ICON, icon);
        }
    }

    public void actionPerformed(ActionEvent e){
        JFileChooser chooser = new JFileChooser();
        chooser.setCurrentDirectory(handler.getCurrentDirectory());
        chooser.addChoosableFileFilter(new ExtensionFileFilter("jpg", "JPEG Image"));
        chooser.addChoosableFileFilter(new ExtensionFileFilter("png", "PNG Image"));
        chooser.setMultiSelectionEnabled(false);
        int value = chooser.showSaveDialog(parent);

        if(value == JFileChooser.APPROVE_OPTION){
            handler.setCurrentDirectory(chooser.getCurrentDirectory());

            FileFilter filter = chooser.getFileFilter();
            File target = chooser.getSelectedFile();
            if(!filter.accept(target) && filter instanceof ExtensionFileFilter){
                target = new File(
                    target.getParentFile(), target.getName() + "." +
                    ((ExtensionFileFilter)filter).getExtension()
                );
            }
            storeImage(target);
        }
    }

    private BufferedImage createImage(){
        MdsPane pane = findMdsPane(mds);

        BufferedImage image = new BufferedImage(pane.getWidth(), pane.getHeight(), BufferedImage.TYPE_INT_ARGB);
        Graphics2D g = image.createGraphics();
        pane.paint(g);

        g.dispose();
        return image;
    }

    private void storeImage(File file){
        String fileName = file.getName();
        String extension = fileName.substring(fileName.lastIndexOf('.') + 1);

        try{
            ImageIO.write(createImage(), extension, file);
        } catch(IOException e){
            JOptionPane.showMessageDialog(
                parent,
                mds.getMessages().get("show.error.io.message") + "\r\n" + e.getMessage(),
                mds.getMessages().get("show.error.dialog.title"),
                JOptionPane.ERROR_MESSAGE
            );
        }
    }

    private MdsPane findMdsPane(Container mds){
        Component[] components = mds.getComponents();
        for(int i = 0; i < components.length; i++){
            if(components[i] instanceof MdsPane){
                return (MdsPane)components[i];
            }
            else if(components[i] instanceof Container){
                return findMdsPane((Container)components[i]);
            }
        }
        return null;
    }
}
