package jp.sourceforge.talisman.mds.cluster;

/*
 * $Id: LanceWilliamsClusteringMethodTest.java 158 2008-07-15 10:45:01Z tama3 $
 */

import jp.sourceforge.talisman.mds.Item;
import jp.sourceforge.talisman.mds.distance.ClusterDistanceAlgorithm;
import jp.sourceforge.talisman.mds.distance.ItemDistanceAlgorithm;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

/**
 * 
 * @author Haruaki Tamada
 * @version $Revision: 158 $ 
 */
public class LanceWilliamsClusteringMethodTest{
    private Item[] items;

    @Before
    public void initialize(){
        items = new Item[5];

        items[0] = new Item("item1", new double[] { 3, 4, });
        items[1] = new Item("item2", new double[] { 1, 1, });
        items[2] = new Item("item3", new double[] { 2, 1, });
        items[3] = new Item("item4", new double[] { 1, 2, });
        items[4] = new Item("item5", new double[] { 4, 4, });
    }

    @Test
    public void testBasic() throws Exception{
        HierarchicalClusteringParameter param = new HierarchicalClusteringParameter(2);
        param.setDistanceCalculator(ItemDistanceAlgorithm.SQUARED_EUCLIDEAN_DISTANCE);
        param.setClusterDistanceCalculator(ClusterDistanceAlgorithm.CENTROID_METHOD);
        ClusteringMethod method = new LanceWilliamsClusteringMethod(param);

        Cluster[] clusters = method.clustering(items);
        Assert.assertEquals(2, clusters.length);
        Assert.assertEquals(3, clusters[0].getSize());
        Assert.assertEquals(2, clusters[1].getSize());

        Assert.assertTrue(clusters[0].isContain(items[1]));
        Assert.assertTrue(clusters[0].isContain(items[2]));
        Assert.assertTrue(clusters[0].isContain(items[3]));

        Assert.assertTrue(clusters[1].isContain(items[0]));
        Assert.assertTrue(clusters[1].isContain(items[4]));
    }

    @Test
    public void testBasic2() throws Exception{
        HierarchicalClusteringParameter param = new HierarchicalClusteringParameter(2);
        param.setDistanceCalculator(ItemDistanceAlgorithm.SQUARED_EUCLIDEAN_DISTANCE);
        param.setClusterDistanceCalculator(ClusterDistanceAlgorithm.CENTROID_METHOD);
        ClusteringMethod method = new LanceWilliamsClusteringMethod((ClusteringParameter)param);

        Cluster[] clusters = method.clustering(items);
        Assert.assertEquals(2, clusters.length);
        Assert.assertEquals(3, clusters[0].getSize());
        Assert.assertEquals(2, clusters[1].getSize());

        Assert.assertTrue(clusters[0].isContain(items[1]));
        Assert.assertTrue(clusters[0].isContain(items[2]));
        Assert.assertTrue(clusters[0].isContain(items[3]));

        Assert.assertTrue(clusters[1].isContain(items[0]));
        Assert.assertTrue(clusters[1].isContain(items[4]));
    }

    @Test(expected=ParameterTypeMismatchException.class)
    public void testParamTypeMismatchException() throws Exception{
        ClusteringMethod method = new LanceWilliamsClusteringMethod(new DummyClusteringParameter());
        method.clustering(items);
    }

    @Test(expected=InvalidParameterException.class)
    public void testCalculatorNotFoundException() throws Exception{
        ClusteringMethod method = new LanceWilliamsClusteringMethod(new HierarchicalClusteringParameter(2));
        method.clustering(items);
    }

    @Test(expected=InvalidParameterException.class)
    public void testParameterException() throws Exception{
        ClusteringMethod method = new LanceWilliamsClusteringMethod(new HierarchicalClusteringParameter(10));
        method.clustering(items);
    }
}
